import { tswCtx } from "./Ctx";
import { g_CurrentContext } from "./Scope";
import { tswStyleRule } from "./elm";
export class tswCtxAttr extends tswCtx {
    constructor(el, attrName, attrVals) {
        super();
        this.el = el;
        this.attrName = attrName;
        this.attrVals = attrVals;
    }
    setup(ctxParent) {
        this.setAttrVal();
        this.addCtxToParent(ctxParent);
    }
    update() {
        if (this.ctxParent === null)
            return;
        this.removeChildren();
        this.setAttrVal();
    }
    setAttrVal() {
        g_CurrentContext.use(this, () => this._setAttrVal());
    }
    _setAttrVal() {
        let result = null;
        const isMultiValueType = this.attrName == "class" || this.attrName == "style";
        const separator = this.attrName == "class" ? ' ' : this.attrName == "style" ? ';' : null;
        addAttrString(this.attrVals);
        if (result != null) {
            this.el.setAttribute(this.attrName, result);
        }
        else {
            this.el.removeAttribute(this.attrName);
        }
        function addAttrString(attrValue) {
            if (attrValue == null || attrValue === false) {
                if (!isMultiValueType)
                    result = null;
            }
            else if (typeof attrValue == "string") {
                if (isMultiValueType) {
                    if (attrValue) {
                        if (result) {
                            result += separator;
                            result += attrValue;
                        }
                        else {
                            result = attrValue;
                        }
                    }
                }
                else {
                    result = attrValue;
                }
            }
            else if (attrValue instanceof tswStyleRule) {
                if (attrValue.propName) {
                    const ruleValue = getRuleValue(attrValue.propValue);
                    if (ruleValue)
                        addAttrString(attrValue.propName + ':' + ruleValue);
                }
            }
            else if (attrValue === true) {
                addAttrString('');
            }
            else if (attrValue instanceof Array) {
                attrValue.forEach(v => addAttrString(v));
            }
            else if (attrValue instanceof Function) {
                const v = attrValue();
                addAttrString(v);
            }
            else if (isPropDefReadable(attrValue)) {
                const v = attrValue.get();
                addAttrString(v);
            }
            else {
                addAttrString(attrValue.toString());
            }
        }
    }
    get dbg_attrName() { return this.attrName; }
    get dbg_el() { return this.el; }
}
function getRuleValue(s) {
    if (s instanceof Function) {
        const v = s();
        return v;
    }
    else if (isPropDefReadable(s)) {
        const v = s.get();
        return v;
    }
    else {
        return s;
    }
}
function isPropDefReadable(v) {
    return v && v.get instanceof Function;
}
