import { privates, tswElement } from './elm';
export class tswRawHtml {
    constructor(value) {
        this.value = value;
    }
}
export class tswElementButton extends tswElement {
    constructor() {
        super('button');
    }
    type(val) {
        this.attr('type', val);
        return this;
    }
}
export class tswElementA extends tswElement {
    constructor() {
        super('a');
        this.href("#");
    }
    href(val) {
        this.attr('href', val);
        return this;
    }
}
export class tswElementImg extends tswElement {
    constructor() {
        super('img');
    }
    src(val) {
        this.attr('src', val);
        return this;
    }
}
export class tswElementWithValueBase extends tswElement {
}
export class tswElementWithValue extends tswElementWithValueBase {
    constructor(tagName, propName) {
        super(tagName);
        this.propName = propName;
    }
    value(propDef) {
        this.propDef = propDef;
        return this;
    }
    [privates.ElementWithValueBase.getValueInfos]() {
        return this.propDef == null ? null : { propName: this.propName, propVal: this.propDef };
    }
}
export class tswElementInput extends tswElementWithValue {
    constructor(type, propName) {
        super('input', propName);
        this.attr('type', type);
    }
}
export class tswElementInputText extends tswElementInput {
    constructor() {
        super('text', 'value');
        this.autocomplete('off');
    }
    placeholder(v) {
        this.attr('placeholder', v);
        return this;
    }
    autocomplete(v) {
        this.attr('autocomplete', v);
        return this;
    }
}
export class tswElementInputCheckboxBase extends tswElementInput {
    constructor(type) {
        super(type, 'checked');
    }
}
export class tswElementInputCheckbox extends tswElementInputCheckboxBase {
    constructor() {
        super('checkbox');
    }
}
export class tswElementInputRadio extends tswElementInputCheckboxBase {
    constructor() {
        super('radio');
    }
}
export class tswElementTextArea extends tswElementWithValue {
    constructor() {
        super('textarea', 'value');
    }
    placeholder(v) {
        this.attr('placeholder', v);
        return this;
    }
}
export class tswElementSelect extends tswElementWithValueBase {
    constructor() {
        super('select');
    }
    value(propDef) {
        if (this.propInfos == null)
            this.propInfos = [];
        this.propInfos.push({ propVal: propDef, propName: 'value' });
        return this;
    }
    selectedIndex(propDef) {
        if (this.propInfos == null)
            this.propInfos = [];
        this.propInfos.push({ propVal: propDef, propName: 'selectedIndex' });
        return this;
    }
    [privates.ElementWithValueBase.getValueInfos]() {
        return this.propInfos;
    }
}
export class tswElementOption extends tswElement {
    constructor() {
        super('option');
    }
    value(val) {
        this.attr('value', val);
        return this;
    }
    selected(val) {
        this.attr('selected', val);
        return this;
    }
}
export class tswElementLabel extends tswElement {
    constructor() {
        super('label');
    }
    forId(id) {
        this.attr('for', id);
        return this;
    }
    forRef(ref) {
        console.warn('obsolete method: tswElementLabel forRef(). replace it with forId()');
        return this;
    }
}
export class tswElementTD extends tswElement {
    colSpan(val) {
        this.attr('colSpan', val);
        return this;
    }
    rowSpan(val) {
        this.attr('rowSpan', val);
        return this;
    }
}
